"""The class implementation for the `how_to_reg` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialHowToRegIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `how_to_reg` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none" fill-rule="evenodd"/><g fill-rule="evenodd"><path d="M9 17l3-2.94c-.39-.04-.68-.06-1-.06-2.67 0-8 1.34-8 4v2h9l-3-3zm2-5c2.21 0 4-1.79 4-4s-1.79-4-4-4-4 1.79-4 4 1.79 4 4 4"/><path d="M15.47 20.5L12 17l1.4-1.41 2.07 2.08 5.13-5.17 1.4 1.41z"/></g></svg>'  # noqa



"""Class implementations for the weekday-related mix-in.
"""

from datetime import datetime

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._type.variable_name_mixin import VariableNameMixIn


class WeekdayMixIn(VariableNameMixIn):
    _year: Int
    _month: Int
    _day: Int

    @property
    @add_debug_info_setting(module_name=__name__)
    def weekday_js(self) -> Int:
        """
        Get a current weekday value. This interface sets the weekday based on the
        JavaScript value as follows:

        - 0 -> Sunday
        - 1 -> Monday
        - 2 -> Tuesday
        - 3 -> Wednesday
        - 4 -> Thursday
        - 5 -> Friday
        - 6 -> Saturday

        Returns
        -------
        weekday : Int
            A current weekday value.

        References
        ----------
        - DateTime class weekday_js and weekday_py properties
            - https://simon-ritchie.github.io/apysc/en/datetime_weekday_js_and_weekday_py.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> datetime_: ap.DateTime = ap.DateTime(year=2022, month=12, day=4)
        >>> datetime_.weekday_js  # Sunday
        Int(0)
        >>> datetime_ = ap.DateTime(year=2022, month=12, day=10)
        >>> datetime_.weekday_js  # Saturday
        Int(6)
        """
        weekday: Int = Int(0)
        weekday_js_val: int = self._get_weekday_js_val_with_attrs()
        weekday._value = weekday_js_val
        self._append_weekday_js_getter_expression(weekday_val=weekday)
        return weekday

    @final
    def _append_weekday_js_getter_expression(self, *, weekday_val: Int) -> None:
        """
        Append a JavaScript weekday's getter expression string.

        Parameters
        ----------
        weekday_val : Int
            A weekday value to use in an expression.
        """
        from apysc._expression import expression_data_util

        expression: str = (
            f"{weekday_val.variable_name} = {self.variable_name}.getDay();"
        )
        expression_data_util.append_js_expression(expression=expression)

    @final
    def _get_weekday_js_val_with_attrs(self) -> int:
        """
        Get a current JavaScript weekday's integer value with the year, month,
        and day attributes.

        Returns
        -------
        weekday_js_val : int
            A current JavaScript weekday's integer value.
        """
        if (
            not hasattr(self, "_year")
            or not hasattr(self, "_month")
            or not hasattr(self, "_day")
        ):
            return 0
        datetime_: datetime = datetime(
            self._year._value, self._month._value, self._day._value
        )
        weekday_js_val: int = datetime_.weekday() + 1
        if weekday_js_val == 7:
            weekday_js_val = 0
        return weekday_js_val

    @property
    @add_debug_info_setting(module_name=__name__)
    def weekday_py(self) -> Int:
        """
        Get a current weekday value. This interface sets the weekday based on the
        Python value as follows:

        - 0 -> Monday
        - 1 -> Thursday
        - 2 -> Wednesday
        - 3 -> Thursday
        - 4 -> Friday
        - 5 -> Saturday
        - 6 -> Sunday

        Returns
        -------
        weekday : Int
            A current weekday value.

        References
        ----------
        - DateTime class weekday_js and weekday_py properties
            - https://simon-ritchie.github.io/apysc/en/datetime_weekday_js_and_weekday_py.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> datetime_: ap.DateTime = ap.DateTime(year=2022, month=12, day=5)
        >>> datetime_.weekday_py  # Monday
        Int(0)
        >>> datetime_: ap.DateTime = ap.DateTime(year=2022, month=12, day=4)
        >>> datetime_.weekday_py  # Sunday
        Int(6)
        """
        weekday: Int = Int(0)
        weekday_py_val: int = self._get_weekday_py_val_with_attrs()
        weekday._value = weekday_py_val
        self._append_weekday_py_getter_expression(weekday_val=weekday)
        return weekday

    @final
    def _append_weekday_py_getter_expression(self, *, weekday_val: Int) -> None:
        """
        Append a Python weekday's getter expression string.

        Parameters
        ----------
        weekday_val : Int
            A weekday value to use in an expression.
        """
        from apysc._expression import expression_data_util

        expression: str = (
            f"{weekday_val.variable_name} = {self.variable_name}.getDay() - 1;"
            f"\nif ({weekday_val.variable_name} === -1) {{"
            f"\n  {weekday_val.variable_name} = 6;"
            "\n}"
        )
        expression_data_util.append_js_expression(expression=expression)

    @final
    def _get_weekday_py_val_with_attrs(self) -> int:
        """
        Append a Python weekday's getter expression string.

        Returns
        -------
        weekday_py_val : int
            A current Python weekday's integer value.
        """
        if (
            not hasattr(self, "_year")
            or not hasattr(self, "_month")
            or not hasattr(self, "_day")
        ):
            return 0
        datetime_: datetime = datetime(
            self._year._value, self._month._value, self._day._value
        )
        return datetime_.weekday()



"""The class implementation for the `closed_caption_off` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialClosedCaptionOffIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `closed_caption_off` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M19.5 5.5v13h-15v-13h15zM19 4H5c-1.11 0-2 .9-2 2v12c0 1.1.89 2 2 2h14c1.1 0 2-.9 2-2V6c0-1.1-.9-2-2-2zm-8 7H9.5v-.5h-2v3h2V13H11v1c0 .55-.45 1-1 1H7c-.55 0-1-.45-1-1v-4c0-.55.45-1 1-1h3c.55 0 1 .45 1 1v1zm7 0h-1.5v-.5h-2v3h2V13H18v1c0 .55-.45 1-1 1h-3c-.55 0-1-.45-1-1v-4c0-.55.45-1 1-1h3c.55 0 1 .45 1 1v1z"/></svg>'  # noqa



"""The class implementation for the outlined `label_off` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialLabelOffOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `label_off` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M16 7l3.55 5-1.63 2.29 1.43 1.43L22 12l-4.37-6.16C17.27 5.33 16.67 5 16 5l-7.37.01 2 1.99H16zM2 4.03l1.58 1.58C3.22 5.96 3 6.46 3 7v10c0 1.1.9 1.99 2 1.99L16 19c.28 0 .55-.07.79-.18L18.97 21l1.41-1.41L3.41 2.62 2 4.03zM14.97 17H5V7.03L14.97 17z"/></svg>'  # noqa



"""The mix-in class implementation for the `SvgText`'s
`_skip_line_color_expression_appending` attribute.
"""

from typing import Optional

from typing_extensions import final

from apysc._color.color import Color
from apysc._html.debug_mode import add_debug_info_setting


class SvgTextSkipLineColorExpAppendingMixIn:
    _skip_line_color_expression_appending: bool = False

    @final
    @add_debug_info_setting(module_name=__name__)
    def _set_line_color_expression_skipping_attr(
        self,
        *,
        line_color: Optional[Color],
    ) -> None:
        """
        Set a boolean attribute, whether to skip a line-color expression's appending
        or not.

        Parameters
        ----------
        line_color : Optional[Color]
            A line-color setting.
        """
        if line_color is None:
            self._skip_line_color_expression_appending = True
            return
        self._skip_line_color_expression_appending = False



"""The class implementation for the `lock_clock` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialLockClockIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `lock_clock` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M14.5 14.2l2.9 1.7-.8 1.3L13 15v-5h1.5v4.2zM22 14c0 4.41-3.59 8-8 8-2.02 0-3.86-.76-5.27-2H4c-1.15 0-2-.85-2-2V9c0-1.12.89-1.96 2-2v-.5C4 4.01 6.01 2 8.5 2c2.34 0 4.24 1.79 4.46 4.08.34-.05.69-.08 1.04-.08 4.41 0 8 3.59 8 8zM6 7h5v-.74C10.88 4.99 9.8 4 8.5 4 7.12 4 6 5.12 6 6.5V7zm14 7c0-3.31-2.69-6-6-6s-6 2.69-6 6 2.69 6 6 6 6-2.69 6-6z"/></svg>'  # noqa



"""Files' common utilities implementations.

Mainly following interfaces are defined:

- empty_directory
    - Empty specified directory.
- read_txt
    - Read specified file's text.
- save_plain_txt
    - Save plain text string to file.
- append_plain_txt
    - Append plain text string to file.
- remove_file_if_exists
    - Remove a specified file if it exists.
- get_abs_directory_path_from_file_path
    - Get an absolute directory path of a specified file.
- get_abs_module_dir_path
    - Get a specified module's absolute directory path.
- get_specified_ext_file_paths_recursively
    - Get specified extension file paths recursively.
- count_files_recursively
    - Count the existing files number in a specified directory.
- delete_file_if_exists
    - Delete a specified file if it exists.
"""

import os
import shutil
import traceback
from types import ModuleType
from typing import List
from typing import Optional


def empty_directory(*, directory_path: str) -> None:
    """
    Empty specified directory.

    Parameters
    ----------
    directory_path : str
        Directory path to empty. This interface does not
        remove this folder itself.
    """
    if os.path.isdir(directory_path):
        shutil.rmtree(directory_path, ignore_errors=True)
    os.makedirs(directory_path, exist_ok=True)


def read_txt(*, file_path: str) -> str:
    """
    Read specified file's text.

    Parameters
    ----------
    file_path : str
        File path to read.

    Returns
    -------
    txt : str
        Target file's text.
    """
    with open(file_path, "r", encoding="utf-8") as f:
        try:
            txt: str = f.read()
        except Exception:
            err_msg: str = (
                f"{traceback.format_exc()}" f"\nFailed the file reading: {file_path}"
            )
            raise Exception(err_msg)
    return txt


def save_plain_txt(*, txt: str, file_path: str) -> None:
    """
    Save plain text string to file.

    Parameters
    ----------
    txt : str
        Plain text string to save.
    file_path : str
        Destination file path.
    """
    dir_path: str = get_abs_directory_path_from_file_path(file_path=file_path)
    os.makedirs(dir_path, exist_ok=True)
    with open(file_path, "w", encoding="utf-8") as f:
        f.write(txt)


def append_plain_txt(*, txt: str, file_path: str) -> None:
    """
    Append plain text string to file.

    Parameters
    ----------
    txt : str
        Plain text string to append.
    file_path : str
        Destination file path.
    """
    dir_path: str = get_abs_directory_path_from_file_path(file_path=file_path)
    os.makedirs(dir_path, exist_ok=True)
    with open(file_path, "a", encoding="utf-8") as f:
        f.write(txt)


def remove_file_if_exists(*, file_path: str) -> None:
    """
    Remove a specified file if it exists.

    Parameters
    ----------
    file_path : str
        File path to remove.
    """
    if not os.path.isfile(file_path):
        return
    os.remove(file_path)


def get_abs_directory_path_from_file_path(*, file_path: str) -> str:
    """
    Get an absolute directory path of a specified file.

    Parameters
    ----------
    file_path : str
        Target file path.

    Returns
    -------
    dir_path : str
        An absolute directory path.
    """
    dir_path: str = os.path.dirname(file_path)
    dir_path += "/"
    return dir_path


def get_abs_module_dir_path(*, module: ModuleType) -> str:
    """
    Get a specified module's absolute directory path.

    Parameters
    ----------
    module : ModuleType
        Target module.

    Returns
    -------
    abs_module_dir_path : str
        Specified module's abosulute directory path.
    """
    abs_module_dir_path: str = os.path.dirname(str(module.__file__))
    abs_module_dir_path += "/"
    return abs_module_dir_path


def get_specified_ext_file_paths_recursively(
    *,
    extension: str,
    dir_path: str = "./",
    excluding_file_names_prefix_list: Optional[List[str]] = None,
    file_paths: Optional[List[str]] = None,
) -> List[str]:
    """
    Get specified extension file paths recursively.

    Parameters
    ----------
    extension : str
        Target file extension (e.g., '.md', 'md', and so on).
    dir_path : str
        Directory path to search files recursively.
    excluding_file_names_prefix_list : list of str or None, optional
        Excluding file names' prefix list.
        If you specify None, all files become the target.
    file_paths : list of str or None
        Current file paths (only used for recursive function calls).

    Returns
    -------
    file_paths : list of str
        File paths that end with target extension.
    """
    if file_paths is None:
        file_paths = []
    if not extension.startswith("."):
        extension = f".{extension}"
    file_or_dir_names: List[str] = os.listdir(dir_path)
    for file_or_dir_name in file_or_dir_names:
        file_or_dir_path: str = os.path.join(dir_path, file_or_dir_name)
        if os.path.isdir(file_or_dir_path):
            file_paths = get_specified_ext_file_paths_recursively(
                extension=extension,
                dir_path=file_or_dir_path,
                excluding_file_names_prefix_list=(excluding_file_names_prefix_list),
                file_paths=file_paths,
            )
            continue
        if not file_or_dir_path.endswith(extension):
            continue

        has_excluding_prefix: bool = _has_excluding_prefix(
            file_name=file_or_dir_name,
            excluding_file_names_prefix_list=excluding_file_names_prefix_list,
        )
        if has_excluding_prefix:
            continue

        file_paths.append(file_or_dir_path)
    return file_paths


def _has_excluding_prefix(
    *, file_name: str, excluding_file_names_prefix_list: Optional[List[str]]
) -> bool:
    """
    Get a boolean indicating whether a specified file name has
    an excluding file name suffix or not.

    Parameters
    ----------
    file_name : str
        A target file name.
    excluding_file_names_prefix_list : Optional[List[str]]
        An excluding file names' prefix list.

    Returns
    -------
    result : bool
        This function returns True if a specified file name has
        an excluding file name suffix.
    """
    if excluding_file_names_prefix_list is None:
        return False
    for excluding_prefix in excluding_file_names_prefix_list:
        if file_name.startswith(excluding_prefix):
            return True
    return False


def count_files_recursively(*, dir_path: str) -> int:
    """
    Count the existing files number in a specified directory
    recursively.

    Parameters
    ----------
    dir_path : str
        Target directory path.

    Returns
    -------
    count : int
        Existing files count.
    """
    if not os.path.isdir(dir_path):
        return 0
    file_or_dir_names: List[str] = os.listdir(dir_path)
    count: int = 0
    for file_or_dir_name in file_or_dir_names:
        file_or_dir_path: str = os.path.join(dir_path, file_or_dir_name)
        if os.path.isdir(file_or_dir_path):
            count += count_files_recursively(dir_path=file_or_dir_path)
            continue
        if os.path.isfile(file_or_dir_path):
            count += 1
    return count


def delete_file_if_exists(*, file_path: str) -> bool:
    """
    Delete a specified file if it exists.

    Parameters
    ----------
    file_path : str
        A target file path.

    Returns
    -------
    is_deleted : bool
        This interface returns True if this interface executes
        deletion.
    """
    if not os.path.isfile(file_path):
        return False
    os.remove(file_path)
    return True



"""The class implementation for the outlined `help` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialHelpOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `help` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M12 2C6.48 2 2 6.48 2 12s4.48 10 10 10 10-4.48 10-10S17.52 2 12 2zm1 17h-2v-2h2v2zm2.07-7.75l-.9.92C13.45 12.9 13 13.5 13 15h-2v-.5c0-1.1.45-2.1 1.17-2.83l1.24-1.26c.37-.36.59-.86.59-1.41 0-1.1-.9-2-2-2s-2 .9-2 2H8c0-2.21 1.79-4 4-4s4 1.79 4 4c0 .88-.36 1.68-.93 2.25z"/></svg>'  # noqa



"""Class implementation for the max-related mix-in.
"""

from typing import Any
from typing import List
from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.array import Array
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._validation import arg_validation_decos


class MaxMixIn:
    @classmethod
    @final
    @add_debug_info_setting(module_name=__name__)
    @arg_validation_decos.is_nums_array(arg_position_index=1)
    def max(
        cls,
        values: Array,
    ) -> Number:
        """
        Get a maximum number from a specified array's values.

        Parameters
        ----------
        values : Array[Union[Int, Number, int, float]]
            An array of numbers.

        Returns
        -------
        max_value : Number
            Maximum number in an array.

        References
        ----------
        - Math max interface
            - https://simon-ritchie.github.io/apysc/en/math_max.html

        Examples
        --------
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> arr: ap.Array = ap.Array([10, 9.5, ap.Int(8), ap.Number(8.5)])
        >>> max_value: ap.Number = ap.Math.max(values=arr)
        >>> max_value
        Number(10.0)
        """
        from apysc._expression import expression_data_util

        max_value_variable_name_suffix: str = (
            _get_max_value_variable_name_suffix_from_arr(arr=values)
        )
        max_value: Number = Number(
            0, variable_name_suffix=max_value_variable_name_suffix
        )
        max_float_value: float = _get_max_float_value(values=values)
        max_value._value = max_float_value
        expression: str = (
            f"{max_value.variable_name} = {values.variable_name}.reduce("
            "function (a, b) {return Math.max(a, b)});"
        )
        expression_data_util.append_js_expression(expression=expression)
        return max_value


def _get_max_value_variable_name_suffix_from_arr(*, arr: Array) -> str:
    """
    Get a maximum value's variable name suffix from a specified array.

    Parameters
    ----------
    arr : Array
        An array of numbers.

    Returns
    -------
    suffix : str
        An extracted variable name suffix.
    """
    from apysc._type.variable_name_suffix_mixin import VariableNameSuffixMixIn

    values: List[float] = [float(value) for value in arr._value]
    max_value: float = max(values)
    max_value_index: int = values.index(max_value)
    max_value_: Any = arr._value[max_value_index]
    if isinstance(max_value_, VariableNameSuffixMixIn):
        return max_value_._variable_name_suffix
    return ""


def _get_max_float_value(*, values: Array[Union[Int, Number, int, float]]) -> float:
    """
    Get a maximum float value from a specified array.

    Parameters
    ----------
    values : Array[Union[Int, Number, int, float]]
        An array of numbers.

    Returns
    -------
    max_value : float
        A maximum float value.
    """
    values_: List[float] = []
    for value in values._value:
        if isinstance(value, (Int, Number)):
            values_.append(float(value._value))
            continue
        values_.append(float(value))
    max_value: float = max(values_)
    return max_value



"""The class implementation for the outlined `contact_support` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialContactSupportOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `contact_support` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M11 23.59v-3.6c-5.01-.26-9-4.42-9-9.49C2 5.26 6.26 1 11.5 1S21 5.26 21 10.5c0 4.95-3.44 9.93-8.57 12.4l-1.43.69zM11.5 3C7.36 3 4 6.36 4 10.5S7.36 18 11.5 18H13v2.3c3.64-2.3 6-6.08 6-9.8C19 6.36 15.64 3 11.5 3zm-1 11.5h2v2h-2zm2-1.5h-2c0-3.25 3-3 3-5 0-1.1-.9-2-2-2s-2 .9-2 2h-2c0-2.21 1.79-4 4-4s4 1.79 4 4c0 2.5-3 2.75-3 5z"/></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: rectangle_geom.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# RectangleGeom class": "# RectangleGeom クラス",
    ##################################################
    "This page explains the `RectangleGeom` class.": "このページでは`RectangleGeom`クラスについて説明します。",  # noqa
    ##################################################
    "## What class is this?": "## クラス概要",
    ##################################################
    "The `RectangleGeom` class has the rectangle's geometry interfaces, such as the `left_x`, `center_x`, `right_x`, `top_y`, `center_y`, `bottom_y`, `width`, and `height`.": "`RectangleGeom`クラスは`lext_x`や`center_x`、`right_x`、`top_y`、`center_y`、`bottom_y`、`width`、`height`といった四角の幾何学（座標やサイズなど）の各インターフェイスを持ちます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "In most cases, the apysc initializes the `RectangleGeom` class internally.": "多くのケースでは、apyscが内部で`RectangleGeom`クラスを初期化します。",  # noqa
    ##################################################
    "For instance, the `get_bounds` method returns a `RectangleGeom` instance; and sets its instance's rectangle geometry data (bounding box).": "例えば、`get_bounds`メソッドは`RectangleGeom`インスタンスを返却し、且つそのインスタンスに四角の幾何学のデータ（バウンディングボックス）を設定します。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=300,\n    stage_height=200,\n    stage_elem_id="stage",\n)\nrectangle: ap.Rectangle = ap.Rectangle(\n    x=50,\n    y=50,\n    width=200,\n    height=75,\n    fill_color=ap.Color("#0af"),\n)\nbounding_box: ap.RectangleGeom = rectangle.get_bounds()\ntext_1: ap.SvgText = ap.SvgText(\n    text=(\n        ap.String("Left x: ")\n        + bounding_box.left_x.to_string()\n        + ap.String(" width: ")\n        + bounding_box.width.to_string()\n    ),\n    x=50,\n    y=150,\n    fill_color=ap.Color("#aaa"),\n)\nap.save_overall_html(dest_dir_path="rectangle_geom_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=300,\n    stage_height=200,\n    stage_elem_id="stage",\n)\nrectangle: ap.Rectangle = ap.Rectangle(\n    x=50,\n    y=50,\n    width=200,\n    height=75,\n    fill_color=ap.Color("#0af"),\n)\nbounding_box: ap.RectangleGeom = rectangle.get_bounds()\ntext_1: ap.SvgText = ap.SvgText(\n    text=(\n        ap.String("Left x: ")\n        + bounding_box.left_x.to_string()\n        + ap.String(" width: ")\n        + bounding_box.width.to_string()\n    ),\n    x=50,\n    y=150,\n    fill_color=ap.Color("#aaa"),\n)\nap.save_overall_html(dest_dir_path="rectangle_geom_basic_usage/")\n```',  # noqa
    ##################################################
    "## Each attribute point": "## 各属性の座標",
    ##################################################
    "The following example shows each attribute point:": "以下の例では各属性の座標を表示しています:",
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=600,\n    stage_height=440,\n    stage_elem_id="stage",\n)\ncircle: ap.Circle = ap.Circle(\n    x=250,\n    y=220,\n    radius=150,\n    fill_color=ap.Color("#0af"),\n)\nbounding_box: ap.RectangleGeom = circle.get_bounds()\n\nLINE_COLOR: ap.Color = ap.Color("#aaa")\nbox_rectangle: ap.Rectangle = ap.Rectangle(\n    x=bounding_box.left_x,\n    y=bounding_box.top_y,\n    width=bounding_box.width,\n    height=bounding_box.height,\n    line_color=LINE_COLOR,\n)\n\nPOINT_RADIUS: int = 10\nfill_color: ap.Color = ap.Color("#fd63c3")\nleft_x_and_top_y_circle: ap.Circle = ap.Circle(\n    x=bounding_box.left_x,\n    y=bounding_box.top_y,\n    radius=POINT_RADIUS,\n    fill_color=fill_color,\n)\nleft_x_and_top_y_text: ap.SvgText = ap.SvgText(\n    text="left_x and top_y",\n    x=bounding_box.left_x,\n    y=bounding_box.top_y - 15,\n    fill_color=fill_color,\n)\n\nfill_color = ap.Color("#ae59e3")\nright_x_and_top_y_circle: ap.Circle = ap.Circle(\n    x=bounding_box.right_x,\n    y=bounding_box.top_y,\n    radius=POINT_RADIUS,\n    fill_color=fill_color,\n)\nright_x_and_top_y_text: ap.SvgText = ap.SvgText(\n    text="right_x and top_y",\n    x=bounding_box.right_x,\n    y=bounding_box.top_y - 15,\n    fill_color=fill_color,\n)\n\nfill_color = ap.Color("#726efa")\nleft_x_and_bottom_y_circle: ap.Circle = ap.Circle(\n    x=bounding_box.left_x,\n    y=bounding_box.bottom_y,\n    radius=POINT_RADIUS,\n    fill_color=fill_color,\n)\nleft_x_and_bottom_y_text: ap.SvgText = ap.SvgText(\n    text="left_x and bottom_y",\n    x=bounding_box.left_x,\n    y=bounding_box.bottom_y + 31,\n    fill_color=fill_color,\n)\n\nfill_color = ap.Color("#6eaee6")\nright_x_and_bottom_y_circle: ap.Circle = ap.Circle(\n    x=bounding_box.right_x,\n    y=bounding_box.bottom_y,\n    radius=POINT_RADIUS,\n    fill_color=fill_color,\n)\nright_x_and_bottom_y_text: ap.SvgText = ap.SvgText(\n    text="right_x and bottom_y",\n    x=bounding_box.right_x,\n    y=bounding_box.bottom_y + 31,\n    fill_color=fill_color,\n)\n\nfill_color = ap.Color("#ffffff")\ncenter_x_and_center_y_circle: ap.Circle = ap.Circle(\n    x=bounding_box.center_x,\n    y=bounding_box.center_y,\n    radius=POINT_RADIUS,\n    fill_color=fill_color,\n)\ncenter_x_and_center_y_text: ap.SvgText = ap.SvgText(\n    text="center_x and center_y",\n    x=bounding_box.center_x + 25,\n    y=bounding_box.center_y + 5,\n    fill_color=fill_color,\n)\n\nap.save_overall_html(dest_dir_path="rectangle_geom_each_attribute_point/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=600,\n    stage_height=440,\n    stage_elem_id="stage",\n)\ncircle: ap.Circle = ap.Circle(\n    x=250,\n    y=220,\n    radius=150,\n    fill_color=ap.Color("#0af"),\n)\nbounding_box: ap.RectangleGeom = circle.get_bounds()\n\nLINE_COLOR: ap.Color = ap.Color("#aaa")\nbox_rectangle: ap.Rectangle = ap.Rectangle(\n    x=bounding_box.left_x,\n    y=bounding_box.top_y,\n    width=bounding_box.width,\n    height=bounding_box.height,\n    line_color=LINE_COLOR,\n)\n\nPOINT_RADIUS: int = 10\nfill_color: ap.Color = ap.Color("#fd63c3")\nleft_x_and_top_y_circle: ap.Circle = ap.Circle(\n    x=bounding_box.left_x,\n    y=bounding_box.top_y,\n    radius=POINT_RADIUS,\n    fill_color=fill_color,\n)\nleft_x_and_top_y_text: ap.SvgText = ap.SvgText(\n    text="left_x and top_y",\n    x=bounding_box.left_x,\n    y=bounding_box.top_y - 15,\n    fill_color=fill_color,\n)\n\nfill_color = ap.Color("#ae59e3")\nright_x_and_top_y_circle: ap.Circle = ap.Circle(\n    x=bounding_box.right_x,\n    y=bounding_box.top_y,\n    radius=POINT_RADIUS,\n    fill_color=fill_color,\n)\nright_x_and_top_y_text: ap.SvgText = ap.SvgText(\n    text="right_x and top_y",\n    x=bounding_box.right_x,\n    y=bounding_box.top_y - 15,\n    fill_color=fill_color,\n)\n\nfill_color = ap.Color("#726efa")\nleft_x_and_bottom_y_circle: ap.Circle = ap.Circle(\n    x=bounding_box.left_x,\n    y=bounding_box.bottom_y,\n    radius=POINT_RADIUS,\n    fill_color=fill_color,\n)\nleft_x_and_bottom_y_text: ap.SvgText = ap.SvgText(\n    text="left_x and bottom_y",\n    x=bounding_box.left_x,\n    y=bounding_box.bottom_y + 31,\n    fill_color=fill_color,\n)\n\nfill_color = ap.Color("#6eaee6")\nright_x_and_bottom_y_circle: ap.Circle = ap.Circle(\n    x=bounding_box.right_x,\n    y=bounding_box.bottom_y,\n    radius=POINT_RADIUS,\n    fill_color=fill_color,\n)\nright_x_and_bottom_y_text: ap.SvgText = ap.SvgText(\n    text="right_x and bottom_y",\n    x=bounding_box.right_x,\n    y=bounding_box.bottom_y + 31,\n    fill_color=fill_color,\n)\n\nfill_color = ap.Color("#ffffff")\ncenter_x_and_center_y_circle: ap.Circle = ap.Circle(\n    x=bounding_box.center_x,\n    y=bounding_box.center_y,\n    radius=POINT_RADIUS,\n    fill_color=fill_color,\n)\ncenter_x_and_center_y_text: ap.SvgText = ap.SvgText(\n    text="center_x and center_y",\n    x=bounding_box.center_x + 25,\n    y=bounding_box.center_y + 5,\n    fill_color=fill_color,\n)\n\nap.save_overall_html(dest_dir_path="rectangle_geom_each_attribute_point/")\n```',  # noqa
    ##################################################
    "## RectangleGeom constructor API": "## RectangleGeom クラスのコンストラクタのAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "The rectangle's geometry class.<hr>": "四角の幾何学情報を扱うためのクラスです。<hr>",
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `left_x`: Number": "- `left_x`: Number",
    ##################################################
    "  - The rectangle left x coordinate.": "  - 四角の左端のX座標。",
    ##################################################
    "- `center_x`: Number": "- `center_x`: Number",
    ##################################################
    "  - The rectangle center x coordinate.": "  - 四角の中央のX座標。",
    ##################################################
    "- `right_x`: Number": "- `right_x`: Number",
    ##################################################
    "  - The rectangle right x coordinate.": "  - 四角の右端のX座標。",
    ##################################################
    "- `top_y`: Number": "- `top_y`: Number",
    ##################################################
    "  - The rectangle top y coordinate.": "  - 四角の上端のY座標。",
    ##################################################
    "- `center_y`: Number": "- `center_y`: Number",
    ##################################################
    "  - The rectangle center y coordinate.": "  - 四角の中央のY座標。",
    ##################################################
    "- `bottom_y`: Number": "- `bottom_y`: Number",
    ##################################################
    "  - The rectangle bottom y coordinate.": "  - 四角の下端のY座標。",
    ##################################################
    "- `width`: Int": "- `width`: Int",
    ##################################################
    "  - The rectangle width.": "  - 四角の幅。",
    ##################################################
    "- `height`: Int": "- `height`: Int",
    ##################################################
    "  - The Rectangle height.": "  - 四角の高さ。",
    ##################################################
    "## RectangleGeom left_x property API": "## RectangleGeom クラスの left_x 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get the rectangle left x coordinate.<hr>": "四角の左端のX座標を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `left_x`: Number": "- `left_x`: Number",
    ##################################################
    "  - The rectangle left x coordinate.": "  - 四角の左端のX座標。",
    ##################################################
    "## RectangleGeom center_x property API": "## RectangleGeom クラスの center_x 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get the rectangle center x coordinate.<hr>": "四角の中央のX座標を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `center_x`: Number": "- `center_x`: Number",
    ##################################################
    "  - The rectangle center x coordinate.": "  - 四角の中央のX座標。",
    ##################################################
    "## RectangleGeom right_x property API": "## RectangleGeom クラスの right_x 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get the rectangle right x coordinate.<hr>": "四角の右端のX座標を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `right_x`: Number": "- `right_x`: Number",
    ##################################################
    "  - The rectangle right x coordinate.": "  - 四角の右端のX座標。",
    ##################################################
    "## RectangleGeom top_y property API": "## RectangleGeom クラスの top_y 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get the rectangle top y coordinate.<hr>": "四角の上端のY座標を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `top_y`: Number": "- `top_y`: Number",
    ##################################################
    "  - The rectangle top y coordinate.": "  - 四角の上端のY座標。",
    ##################################################
    "## RectangleGeom center_y property API": "## RectangleGeom クラスの center_y 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get the rectangle center y coordinate.<hr>": "四角の中央のY座標を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `center_y`: Number": "- `center_y`: Number",
    ##################################################
    "  - The rectangle center y coordinate.": "  - 四角の中央のY座標。",
    ##################################################
    "## RectangleGeom bottom_y property API": "## RectangleGeom クラスの bottom_y 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get the rectangle bottom y coordinate.<hr>": "四角の下端のY座標を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `bottom_y`: Number": "- `bottom_y`: Number",
    ##################################################
    "  - The rectangle bottom y coordinate.": "  - 四角の下端のY座標。",
    ##################################################
    "## RectangleGeom width property API": "## RectangleGeom クラスの width 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get the rectangle geometry width.<hr>": "四角の幅の値を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `width`: Int": "- `width`: Int",
    ##################################################
    "  - The rectangle geometry width.": "  - 四角の幅の値。",
    ##################################################
    "## RectangleGeom height property API": "## RectangleGeom の height 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get the rectangle geometry height.<hr>": "四角の高さの値を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `height`: Int": "- `height`: Int",
    ##################################################
    "  - The rectangle geometry height.": "  - 四角の高さの値。",
    ##################################################
    "## get_bounds method API": "## get_bounds メソッドのAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get an instance's bounding-box geometry data.<hr>": "インスタンスのバウンディングボックスの幾何データを取得します。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `target_coordinate_space_object`: DisplayObject or None, default None": "- `target_coordinate_space_object`: DisplayObject or None, default None",  # noqa
    ##################################################
    "  - Target coordinate space object. If None is specified, then this method returns the bounding-box data based on the stage. If a `DisplayObject` instance is specified, then this method returns the bounding-box data based on the specified object.": "  - 座標の基準となるオブジェクト。もしNoneが指定された場合、このメソッドはステージを基準としたバウンディングボックスのデータを返却します。もし`DisplayObject`のインスタンスが指定された場合、ごのメソッドは指定されたオブジェクトの座標を基準としたバウンディングボックスのデータを返却します。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `bounding_box`: RectangleGeom": "- `bounding_box`: RectangleGeom",
    ##################################################
    "  - An instance's bounding-box geometry data.": "  - インスタンスのバウンディングボックスの幾何データ。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage(\n...     background_color=ap.Color("#333"), stage_width=250, stage_height=350\n... )\n>>> rectangle: ap.Rectangle = ap.Rectangle(\n...     x=50,\n...     y=100,\n...     width=150,\n...     height=200,\n...     fill_color=ap.Color("#0af"),\n... )\n>>> bounding_box: ap.RectangleGeom = rectangle.get_bounds()\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage(\n...     background_color=ap.Color("#333"), stage_width=250, stage_height=350\n... )\n>>> rectangle: ap.Rectangle = ap.Rectangle(\n...     x=50,\n...     y=100,\n...     width=150,\n...     height=200,\n...     fill_color=ap.Color("#0af"),\n... )\n>>> bounding_box: ap.RectangleGeom = rectangle.get_bounds()\n```',  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [get_bounds interface](https://simon-ritchie.github.io/apysc/en/get_bounds.md)": "- [get_bounds インターフェイス](https://simon-ritchie.github.io/apysc/jp/jp_get_bounds.md)",  # noqa
}



"""The class implementation for the outlined `batch_prediction` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialBatchPredictionOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `batch_prediction` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24" x="0"/><path d="M17,8H7c-1.1,0-2,0.9-2,2v10c0,1.1,0.9,2,2,2h10c1.1,0,2-0.9,2-2V10C19,8.9,18.1,8,17,8z M13,20.5h-2V19h2V20.5z M13,18h-2 c0-1.5-2.5-3-2.5-5c0-1.93,1.57-3.5,3.5-3.5c1.93,0,3.5,1.57,3.5,3.5C15.5,15,13,16.5,13,18z M18,6.5H6v0C6,5.67,6.67,5,7.5,5h9 C17.33,5,18,5.67,18,6.5L18,6.5z M17,3.5H7v0C7,2.67,7.67,2,8.5,2h7C16.33,2,17,2.67,17,3.5L17,3.5z"/></g></svg>'  # noqa



"""The class implementation for the outlined `speaker_phone` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialSpeakerPhoneOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `speaker_phone` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M7 7.07L8.43 8.5c.91-.91 2.18-1.48 3.57-1.48s2.66.57 3.57 1.48L17 7.07C15.72 5.79 13.95 5 12 5s-3.72.79-5 2.07zM12 1C8.98 1 6.24 2.23 4.25 4.21l1.41 1.41C7.28 4 9.53 3 12 3s4.72 1 6.34 2.62l1.41-1.41C17.76 2.23 15.02 1 12 1zm2.86 9.01L9.14 10C8.51 10 8 10.51 8 11.14v9.71c0 .63.51 1.14 1.14 1.14h5.71c.63 0 1.14-.51 1.14-1.14v-9.71c.01-.63-.5-1.13-1.13-1.13zM15 20H9v-8h6v8z"/></svg>'  # noqa



"""The class implementation for the outlined `find_in_page` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialFindInPageOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `find_in_page` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M14 2H6c-1.1 0-1.99.9-1.99 2L4 20c0 1.1.89 2 1.99 2H18c1.1 0 2-.9 2-2V8l-6-6zM6 4h7l5 5v8.58l-1.84-1.84c1.28-1.94 1.07-4.57-.64-6.28C14.55 8.49 13.28 8 12 8c-1.28 0-2.55.49-3.53 1.46-1.95 1.95-1.95 5.11 0 7.05.97.97 2.25 1.46 3.53 1.46.96 0 1.92-.28 2.75-.83L17.6 20H6V4zm8.11 11.1c-.56.56-1.31.88-2.11.88s-1.55-.31-2.11-.88c-.56-.56-.88-1.31-.88-2.11s.31-1.55.88-2.11c.56-.57 1.31-.88 2.11-.88s1.55.31 2.11.88c.56.56.88 1.31.88 2.11s-.31 1.55-.88 2.11z"/></svg>'  # noqa



"""The class implementation for the `mark_as_unread` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialMarkAsUnreadIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `mark_as_unread` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M18.83 7h-2.6L10.5 4 4 7.4V17c-1.1 0-2-.9-2-2V7.17c0-.53.32-1.09.8-1.34L10.5 2l7.54 3.83c.43.23.73.7.79 1.17zM20 8H7c-1.1 0-2 .9-2 2v9c0 1.1.9 2 2 2h13c1.1 0 2-.9 2-2v-9c0-1.1-.9-2-2-2zm0 3.67L13.5 15 7 11.67V10l6.5 3.33L20 10v1.67z"/></svg>'  # noqa



"""Each display's validation implementations.

Mainly following interfaces are defined:

- validate_stage
    Validate whether the specified instance is Stage type or not.
- validate_display_object
    Validate whether a specified instance is the
    `DisplayObject` type or its subclass type (e.g., `Sprite`).
- validate_display_object_container
    Validate whether a specified instance is a container type
    of a `DisplayObject` instance (e.g., `Sprite`, `Stage`).
- validate_sprite
    Validate specified instance is Sprite type.
- validate_graphics
    Validate specified instance is Graphics type.
- validate_line_cap
    Validate specified line cap style setting.
- validate_line_joints
    Validate specified line joints style setting.
- validate_multiple_line_settings_are_not_set
    Validate that there are no multiple line settings
    (dotted, dashed, and so on).
- validate_css_interface
    Validate whether the specified instance is the `CssInterface`.
"""

from typing import Any
from typing import List

from apysc._display.child_mixin import ChildMixIn
from apysc._display.css_interface import CssInterface


def validate_stage(*, stage: Any) -> None:
    """
    Validate whether the specified instance is Stage type or not.

    Parameters
    ----------
    stage : Stage
        Stage instance to check.

    Raises
    ------
    ValueError
        If a specified instance is not stage type.
    """
    from apysc._display.stage import Stage

    if isinstance(stage, Stage):
        return
    raise ValueError(f"Specified instance is not Stage type: {type(stage)}")


def validate_display_object(
    *, display_object: Any, additional_err_msg: str = ""
) -> None:
    """
    Validate whether a  specified instance is the
    `DisplayObject` type or its subclass type (e.g., Sprite).

    Parameters
    ----------
    display_object : DisplayObject
        A `DisplayObject` instance to check.
    additional_err_msg : str, optional
        An additional error message to display.

    Raises
    ------
    ValueError
        If a specified instance is not `DisplayObject` type
        or its subclass type.
    """
    from apysc._display.display_object import DisplayObject
    from apysc._validation import validation_common_utils

    if isinstance(display_object, DisplayObject):
        return
    err_msg: str = (
        "A specified instance is not DisplayObject or it's subclass type: "
        f"{type(display_object)}"
    )
    err_msg = validation_common_utils.append_additional_err_msg(
        err_msg=err_msg, additional_err_msg=additional_err_msg
    )
    raise ValueError(err_msg)


def validate_display_object_container(
    *, container_object: Any, additional_err_msg: str = ""
) -> ChildMixIn:
    """
    Validate whether a specified instance is a container type
    of a `DisplayObject` instance (e.g., `Sprite`, `Stage`).

    Parameters
    ----------
    container_object : Any
        A target container instance to check.
    additional_err_msg : str, optional
        An additional error message to display.

    Returns
    -------
    container_object : ChildMixIn
        A target container instance.

    Raises
    ------
    ValueError
        If a specified instance is not a container type instance.
    """
    from apysc._validation import validation_common_utils

    if isinstance(container_object, ChildMixIn):
        return container_object
    err_msg: str = (
        "A specified instance is not a container type (e.g., "
        f"`Sprite`, `Stage`) instance: {type(container_object)}"
    )
    err_msg = validation_common_utils.append_additional_err_msg(
        err_msg=err_msg, additional_err_msg=additional_err_msg
    )
    raise ValueError(err_msg)


def validate_sprite(*, sprite: Any) -> None:
    """
    Validate specified instance is Sprite type.

    Parameters
    ----------
    sprite : Sprite
        Sprite instance to check.

    Raises
    ------
    ValueError
        If a specified instance is not Sprite type.
    """
    from apysc._display.sprite import Sprite

    if isinstance(sprite, Sprite):
        return
    raise ValueError(f"Specified instance is not Sprite type: {type(sprite)}")


def validate_graphics(*, graphics: Any) -> None:
    """
    Validate specified instance is Graphics type.

    Parameters
    ----------
    graphics : Graphics
        Graphics instance to check.

    Raises
    ------
    ValueError
        If a specified instance is not Graphics type.
    """
    from apysc._display.graphics import Graphics

    if isinstance(graphics, Graphics):
        return
    raise ValueError(f"Specified instance is not Graphics type: {type(graphics)}")


def validate_line_cap(*, cap: Any, additional_err_msg: str = "") -> None:
    """
    Validate specified line cap style setting.

    Parameters
    ----------
    cap : LineCaps or String
        Target line cap style setting to check.
    additional_err_msg : str, optional
        An additional error message to display.

    Raises
    ------
    ValueError
        If a specified cap setting type is not LineCaps or not defined
        string value.
    """
    from apysc._display.line_caps import LineCaps
    from apysc._type.string import String
    from apysc._validation import validation_common_utils

    if isinstance(cap, LineCaps):
        return
    if isinstance(cap, String):
        for line_cap in LineCaps:
            if line_cap.value == cap._value:
                return
        err_msg: str = (
            f"Not defined cap string is specified: {cap}"
            "\nPlease see definitions of LineCaps."
        )
        err_msg = validation_common_utils.append_additional_err_msg(
            err_msg=err_msg, additional_err_msg=additional_err_msg
        )
        raise ValueError(err_msg)
    err_msg = f"Specified cap style type is not LineCaps or String one: {type(cap)}"
    err_msg = validation_common_utils.append_additional_err_msg(
        err_msg=err_msg, additional_err_msg=additional_err_msg
    )
    raise ValueError(err_msg)


def validate_line_joints(*, joints: Any, additional_err_msg: str = "") -> None:
    """
    Validate specified line joints style setting.

    Parameters
    ----------
    joints : LineJoints or String
        Target line joints style setting to check.
    additional_err_msg : str, optional
        An additional error message to display.

    Raises
    ------
    ValueError
        If specified joints setting type is not LineJoints or not defined
        string value.
    """
    from apysc._display.line_joints import LineJoints
    from apysc._type.string import String
    from apysc._validation import validation_common_utils

    if isinstance(joints, LineJoints):
        return
    if isinstance(joints, String):
        for line_joints in LineJoints:
            if line_joints.value == joints._value:
                return
        err_msg: str = (
            f"Not defined joints string is specified: {joints}"
            "\nPlease see definitions of LineJoints."
        )
        err_msg = validation_common_utils.append_additional_err_msg(
            err_msg=err_msg, additional_err_msg=additional_err_msg
        )
        raise ValueError(err_msg)

    err_msg = (
        "Specified joints type is not LineJoints or String one: " f"{type(joints)}"
    )
    err_msg = validation_common_utils.append_additional_err_msg(
        err_msg=err_msg, additional_err_msg=additional_err_msg
    )
    raise ValueError(err_msg)


def validate_multiple_line_settings_are_not_set(
    *, any_instance: Any, additional_err_msg: str = ""
) -> None:
    """
    Validate that there are no multiple line settings
    (dotted, dashed, and so on).

    Parameters
    ----------
    any_instance : Any
        Any instance to check.
    additional_err_msg : str, optional
        An additional error message to display.

    Raises
    ------
    ValueError
        If there are multiple line settings.
    """
    from apysc._display.line_dash_dot_setting_mixin import LineDashDotSettingMixIn
    from apysc._display.line_dash_setting_mixin import LineDashSettingMixIn
    from apysc._display.line_dot_setting_mixin import LineDotSettingMixIn
    from apysc._display.line_round_dot_setting_mixin import LineRoundDotSettingMixIn
    from apysc._display.line_style_mixin import LineStyleMixIn
    from apysc._validation import validation_common_utils

    valid_setting_names: List[str] = []
    if isinstance(any_instance, (LineStyleMixIn, LineDotSettingMixIn)):
        if (
            hasattr(any_instance, "_line_dot_setting")
            and any_instance._line_dot_setting is not None
        ):
            valid_setting_names.append("LineDotSetting")
    if isinstance(any_instance, (LineStyleMixIn, LineDashSettingMixIn)):
        if (
            hasattr(any_instance, "_line_dash_setting")
            and any_instance._line_dash_setting is not None
        ):
            valid_setting_names.append("LineDashSetting")
    if isinstance(any_instance, (LineStyleMixIn, LineRoundDotSettingMixIn)):
        if (
            hasattr(any_instance, "_line_round_dot_setting")
            and any_instance._line_round_dot_setting is not None
        ):
            valid_setting_names.append("LineRoundDotSetting")
    if isinstance(any_instance, (LineStyleMixIn, LineDashDotSettingMixIn)):
        if (
            hasattr(any_instance, "_line_dash_dot_setting")
            and any_instance._line_dash_dot_setting is not None
        ):
            valid_setting_names.append("LineDashDotSetting")
    if len(valid_setting_names) < 2:
        return
    err_msg: str = (
        "Multiple line settings can not be set."
        f" Current settings: {valid_setting_names}"
    )
    err_msg = validation_common_utils.append_additional_err_msg(
        err_msg=err_msg, additional_err_msg=additional_err_msg
    )
    raise ValueError(err_msg)


def validate_css_interface(*, instance: Any) -> CssInterface:
    """
    Validate whether the specified instance is the `CssInterface`.

    Parameters
    ----------
    instance : Any
        An instance to check.

    Returns
    -------
    instance : CssInterface
        A validated instance.
    """
    if not isinstance(instance, CssInterface):
        raise TypeError(
            "The specified instance is not the `CssInterface` type: "
            f"{type(instance).__name__}."
        )
    return instance



"""This module is for the translation mapping data of the
following document:

Document file: assert_true_and_false.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# assert_true and assert_false interfaces": "# assert_true と assert_false インターフェイス",  # noqa
    ##################################################
    "This page explains the `assert_true` and `assert_false` function interfaces.": "このページでは`assert_true`と`assert_false`関数の各インターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interfaces are these?": "## 各インターフェイスの概要",
    ##################################################
    "The `assert_true` function interface asserts a specified `Boolean` value is true. Conversely, the `assert_false` function interface asserts a specified `Boolean` value is false.": "`assert_true`関数のインターフェイスは指定された`Boolean`の値が真（true）であることをチェックします。逆に`assert_false`関数のインターフェイスは指定された`Boolean`の値が偽（false）であることをチェックします。",  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "- [JavaScript assertion interface basic behavior](assertion_basic_behavior.md)": "- [JavaScriptの各アサーションのインターフェイスの基本的な挙動](jp_assertion_basic_behavior.md)",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `assert_true` and `assert_false` interfaces require `value` argument. The `type_strict` and `msg` arguments are optional (default value of the `type_strict` argument is `True`).": "`assert_true`と`assert_false`の各インターフェイスは`value`引数を必要とします。`type_strict`と`msg`引数は省略可です。`type_strict`引数のデフォルト値は`True`となります。",  # noqa
    ##################################################
    "If the `type_strict` argument is `True`, the assertion will use the JavaScript strict comparison operator (`===`). For instance, if the `value` is `Int(1)` and the `type_strict` is `True`, an assertion will fail (because of the comparison between the `Boolean` and `Int`). Conversely, if the `type_strict` is `False`, `Int(1)` will pass the `assert_true` assertion.": "もしも`type_strict`引数に`True`が指定された場合チェック処理はJavaScriptの厳密な型の比較（`===`による比較）によって行われます。たとえばもし`value`引数の値が`Int(1)`で且つ`type_strict`引数が`True`の場合チェック処理は真偽値と整数（`Int`）間の比較となるため失敗します。逆に`type_strict`が`False`で且つ値が`Int(1)`であれば`assert_true`関数によるチェック処理は通ります。",  # noqa
    ##################################################
    "These interfaces display an assertion result on the browser console.": "これらのインターフェイスによるチェック結果はブラウザ上のコンソールに表示されます。",  # noqa
    ##################################################
    "The following assertion example (`assert_true` and value is the `Boolean(True)`) passes:": "以下の`assert_true`関数と`Boolean(True)`の値を使用した処理のコード例ではチェックを通ります:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\nbool_1: ap.Boolean = ap.Boolean(True)\nap.assert_true(bool_1, msg="Boolean value is not True!")\n\nap.save_overall_html(dest_dir_path="assert_true_basic_usage_1/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\nbool_1: ap.Boolean = ap.Boolean(True)\nap.assert_true(bool_1, msg="Boolean value is not True!")\n\nap.save_overall_html(dest_dir_path="assert_true_basic_usage_1/")\n```',  # noqa
    ##################################################
    "```\n[assert_true]\nRight-side variable name: b_3\nLeft value: true right value: true\n```": "```\n[assert_true]\nRight-side variable name: b_3\nLeft value: true right value: true\n```",  # noqa
    ##################################################
    "The following assertion example (`assert_true` and value is the `Boolean(False)`) fails:": "以下の`assert_true`関数と`Boolean(False)`の値を使った処理ではチェックが失敗します:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\nbool_1: ap.Boolean = ap.Boolean(False)\nap.assert_true(bool_1, msg="Boolean value is not True!")\n\nap.save_overall_html(dest_dir_path="assert_true_basic_usage_2/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\nbool_1: ap.Boolean = ap.Boolean(False)\nap.assert_true(bool_1, msg="Boolean value is not True!")\n\nap.save_overall_html(dest_dir_path="assert_true_basic_usage_2/")\n```',  # noqa
    ##################################################
    "```\n[assert_true]\nRight-side variable name: b_3\nLeft value: true right value: false\n...\nAssertion failed: Boolean value is not True!\n```": "```\n[assert_true]\nRight-side variable name: b_3\nLeft value: true right value: false\n...\nAssertion failed: Boolean value is not True!\n```",  # noqa
    ##################################################
    "The following assertion example (`assert_true` and value is the `Int(1)` and `type_strict` is `True`) will fail:": "以下の`assert_true`関数と`Int(1)`の値を使い、`type_strict`に`True`を指定した例ではチェックが失敗します:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\nint_1: ap.Int = ap.Int(1)\nap.assert_true(int_1, type_strict=True, msg="Value is not Boolean(True)!")\n\nap.save_overall_html(dest_dir_path="assert_true_basic_usage_3/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\nint_1: ap.Int = ap.Int(1)\nap.assert_true(int_1, type_strict=True, msg="Value is not Boolean(True)!")\n\nap.save_overall_html(dest_dir_path="assert_true_basic_usage_3/")\n```',  # noqa
    ##################################################
    "```\n[assert_true]\nRight-side variable name: i_11\nLeft value: true right value: 1\n...\nAssertion failed: Value is not Boolean(True)!\n```": "```\n[assert_true]\nRight-side variable name: i_11\nLeft value: true right value: 1\n...\nAssertion failed: Value is not Boolean(True)!\n```",  # noqa
    ##################################################
    "The following assertion example (`assert_true` and value is the `Int(1)` and `type_strict` is `False`) will pass:": "以下の`assert_true`関数と`Int(1)`の値を使い`type_strict`に`False`を設定した処理ではチェックが通ります:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\nint_1: ap.Int = ap.Int(1)\nap.assert_true(int_1, type_strict=False, msg="Value is not True!")\n\nap.save_overall_html(dest_dir_path="assert_true_basic_usage_4/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\nint_1: ap.Int = ap.Int(1)\nap.assert_true(int_1, type_strict=False, msg="Value is not True!")\n\nap.save_overall_html(dest_dir_path="assert_true_basic_usage_4/")\n```',  # noqa
    ##################################################
    "```\n[assert_true]\nRight-side variable name: i_11\nLeft value: true right value: 1\n```": "```\n[assert_true]\nRight-side variable name: i_11\nLeft value: true right value: 1\n```",  # noqa
    ##################################################
    "The following assertion example (`assert_false` and value is the `Boolean(False)`) passes:": "以下の`assert_false`関数と`Boolean(False)`の値を使った処理ではチェックを通ります:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\nbool_1: ap.Boolean = ap.Boolean(False)\nap.assert_false(bool_1, msg="Value is not False!")\n\nap.save_overall_html(dest_dir_path="assert_false_basic_usage_1/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\nbool_1: ap.Boolean = ap.Boolean(False)\nap.assert_false(bool_1, msg="Value is not False!")\n\nap.save_overall_html(dest_dir_path="assert_false_basic_usage_1/")\n```',  # noqa
    ##################################################
    "```\n[assert_false]\nRight-side variable name: b_3\nLeft value: false right value: false\n```": "```\n[assert_false]\nRight-side variable name: b_3\nLeft value: false right value: false\n```",  # noqa
    ##################################################
    "## assert_true API": "## assert_true API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "JavaScript assertion interface for the true condition.<hr>": "真偽値の真の値のためのJavaScript上のアサーションのインターフェイスです。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `value`: *": "- `value`: *",
    ##################################################
    "  - Target value to check.": "  - チェック対象の値。",
    ##################################################
    "- `type_strict`: bool, default True": "- `type_strict`: bool, default True",
    ##################################################
    "  - Whether strictly check actual value or not. For example, if type_strict is True, an integer of 1 fails tests. On the contrary (if type_strict is False), an integer of 1 passes tests.": "  - 厳密な型でのチェックを行うかどうかの設定です。たとえばtype_strictにTrueを指定した場合は整数の1ではテストは失敗します。逆にtype_strictがFalseの場合整数の1はテストを通過します。",  # noqa
    ##################################################
    "- `msg`: str, optional": "- `msg`: str, optional",
    ##################################################
    "  - Message to display when assertion failed.": "  - チェックに失敗した際に表示するメッセージ。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> int_val: ap.Int = ap.Int(10)\n>>> boolean: ap.Boolean = int_val == 10\n>>> ap.assert_true(boolean)\n```": "```py\n>>> import apysc as ap\n>>> int_val: ap.Int = ap.Int(10)\n>>> boolean: ap.Boolean = int_val == 10\n>>> ap.assert_true(boolean)\n```",  # noqa
    ##################################################
    "## assert_false API": "## assert_false API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "JavaScript assertion interface for the false condition.<hr>": "真偽値の偽の値のためのJavaScript上のアサーションのインターフェイスです。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `value`: *": "- `value`: *",
    ##################################################
    "  - Target value to check.": "  - チェック対象の値。",
    ##################################################
    "- `type_strict`: bool, default True": "- `type_strict`: bool, default True",
    ##################################################
    "  - Whether strictly check actual value or not. For example, if type_strict is True, an integer of 0 fails tests. On the contrary (if type_strict is False), an integer of 0 passes tests.": "  - 厳密な型でのチェックを行うかどうかの設定です。たとえばtype_strictにTrueを指定した場合は整数の0ではテストは失敗します。逆にtype_strictがFalseの場合整数の0はテストを通過します。",  # noqa
    ##################################################
    "- `msg`: str, optional": "- `msg`: str, optional",
    ##################################################
    "  - Message to display when assertion failed.": "  - チェックに失敗した際に表示するメッセージ。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> int_val: ap.Int = ap.Int(10)\n>>> boolean: ap.Boolean = int_val == 11\n>>> ap.assert_false(boolean)\n```": "```py\n>>> import apysc as ap\n>>> int_val: ap.Int = ap.Int(10)\n>>> boolean: ap.Boolean = int_val == 11\n>>> ap.assert_false(boolean)\n```",  # noqa
}



"""Class implementation for the skew x mix-in.
"""

from typing import Dict
from typing import Optional

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.attr_linking_mixin import AttrLinkingMixIn
from apysc._type.int import Int
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.variable_name_mixin import VariableNameMixIn
from apysc._type.variable_name_suffix_attr_or_var_mixin import (
    VariableNameSuffixAttrOrVarMixIn,
)
from apysc._validation import arg_validation_decos


class SkewXMixIn(
    VariableNameSuffixAttrOrVarMixIn,
    VariableNameMixIn,
    RevertMixIn,
    AttrLinkingMixIn,
):
    _skew_x: Int

    @final
    def _initialize_skew_x_if_not_initialized(self) -> None:
        """
        Initialize the _skew_x attribute if this instance does not
        initialize it yet.
        """
        if hasattr(self, "_skew_x"):
            return
        suffix: str = self._get_attr_or_variable_name_suffix(value_identifier="skew_x")
        self._skew_x = Int(
            0,
            variable_name_suffix=suffix,
            skip_init_substitution_expression_appending=True,
        )

        self._append_skew_x_attr_linking_setting()

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_skew_x_attr_linking_setting(self) -> None:
        """
        Append a skew-x attribute linking settings.
        """
        self._append_applying_new_attr_val_exp(
            new_attr=self._skew_x, attr_name="skew_x"
        )
        self._append_attr_to_linking_stack(attr=self._skew_x, attr_name="skew_x")

    @property
    @add_debug_info_setting(module_name=__name__)
    def skew_x(self) -> Int:
        """
        Get a current skew x value of the instance.

        Returns
        -------
        skew_x : Int
            Current skew x value of this instance.

        References
        ----------
        - GraphicsBase skew_x and skew_y interfaces
            - https://simon-ritchie.github.io/apysc/en/graphics_base_skew.html

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color=ap.Color("#0af"))
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50
        ... )
        >>> rectangle.skew_x = ap.Int(50)
        >>> rectangle.skew_x
        Int(50)
        """
        from apysc._type import value_util

        self._initialize_skew_x_if_not_initialized()
        return value_util.get_copy(value=self._skew_x)

    @skew_x.setter
    @arg_validation_decos.is_apysc_num(arg_position_index=1)
    @add_debug_info_setting(module_name=__name__)
    def skew_x(self, value: Int) -> None:
        """
        Update a skew x value of this instance.

        Parameters
        ----------
        value : Int
            Skew x value to set.

        References
        ----------
        - GraphicsBase skew_x and skew_y interfaces
            - https://simon-ritchie.github.io/apysc/en/graphics_base_skew.html
        """
        self._initialize_skew_x_if_not_initialized()
        before_value: Int = self._skew_x
        self._skew_x = value
        self._append_skew_x_update_expression(before_value=before_value)

        self._append_skew_x_attr_linking_setting()

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_skew_x_update_expression(self, *, before_value: Int) -> None:
        """
        Append the skew x updating expression.

        Parameters
        ----------
        before_value : ap.Int
            Before updating value.
        """
        from apysc._expression import expression_data_util
        from apysc._type import value_util

        before_value_str: str = value_util.get_value_str_for_expression(
            value=before_value
        )
        after_value_str: str = value_util.get_value_str_for_expression(
            value=self._skew_x
        )
        expression: str = (
            f"{self.variable_name}.skew(-{before_value_str}, 0);"
            f"\n{self.variable_name}.skew({after_value_str}, 0);"
            f"\n{before_value_str} = {after_value_str};"
        )
        expression_data_util.append_js_expression(expression=expression)

    _skew_x_snapshots: Optional[Dict[str, int]] = None

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make a value snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_skew_x_if_not_initialized()
        self._set_single_snapshot_val_to_dict(
            dict_name="_skew_x_snapshots",
            value=int(self._skew_x._value),
            snapshot_name=snapshot_name,
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert a value if a snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._skew_x._value = self._get_snapshot_val_if_exists(
            current_value=self._skew_x._value,
            snapshot_dict=self._skew_x_snapshots,
            snapshot_name=snapshot_name,
        )



"""Class implementation for the prevent_default mix-in.
"""

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.variable_name_mixin import VariableNameMixIn


class PreventDefaultMixIn(VariableNameMixIn):
    @final
    @add_debug_info_setting(module_name=__name__)
    def prevent_default(self) -> None:
        """
        Prevent the event's default behavior.

        References
        ----------
        - Event class prevent_default and stop_propagation interfaces
            - https://simon-ritchie.github.io/apysc/en/event_prevent_default_and_stop_propagation.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> def on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:
        ...     e.prevent_default()
        ...     rectangle: ap.Rectangle = e.this
        ...     rectangle.fill_color = ap.Color("#f0a")
        ...     rectangle.unbind_mouseup_all()
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color=ap.Color("#0af"))
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50
        ... )
        >>> _ = rectangle.click(on_click)
        """
        from apysc._expression import expression_data_util

        expression: str = f"{self.variable_name}.preventDefault();"
        expression_data_util.append_js_expression(expression=expression)



"""The class implementation for the `wifi_protected_setup` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialWifiProtectedSetupIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `wifi_protected_setup` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><g><g><path d="M16.71,5.29L19,3h-8v8l2.3-2.3c1.97,1.46,3.25,3.78,3.25,6.42c0,1.31-0.32,2.54-0.88,3.63c2.33-1.52,3.88-4.14,3.88-7.13 C19.55,9.1,18.44,6.85,16.71,5.29z"/></g><g><path d="M7.46,8.88c0-1.31,0.32-2.54,0.88-3.63C6,6.77,4.46,9.39,4.46,12.38c0,2.52,1.1,4.77,2.84,6.33L5,21h8v-8l-2.3,2.3 C8.74,13.84,7.46,11.52,7.46,8.88z"/></g></g></g></svg>'  # noqa



"""The class implementations for the initial substitution expression
mix-in.
"""

from abc import ABC
from abc import abstractmethod

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting


class InitialSubstitutionExpMixIn(ABC):
    @abstractmethod
    def _create_initial_substitution_expression(self) -> str:
        """
        Create an initial value's substitution expression string.
        """

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_initial_substitution_expression_if_in_handler_scope(
        self,
        skip_appending: bool,
    ) -> None:
        """
        Append an initial value's expression if a current scope is
        in a handler scope.

        Parameters
        ----------
        skip_appending : bool
            A boolean indicates whether to skip appending an expression
            or not.
        """
        from apysc._expression import event_handler_scope
        from apysc._expression import expression_data_util

        if skip_appending:
            return

        if not event_handler_scope.current_scope_is_in_event_handler():
            return
        expression: str = self._create_initial_substitution_expression()
        expression_data_util.append_js_expression(expression=expression)



"""The mix-in class implementation for the `set_overflow_visible_setting_mixin`.
"""

from typing_extensions import final

from apysc._display.css_interface import CssInterface
from apysc._html.debug_mode import add_debug_info_setting


class SetOverflowVisibleSettingMixIn(CssInterface):
    @final
    @add_debug_info_setting(module_name=__name__)
    def _set_overflow_visible_setting(self) -> None:
        """
        Set the `visible` value to the `overflow` CSS property.
        """
        self.set_css(name="overflow", value="visible")



"""Class implementation for the `revert`-related mix-in.
"""

from typing import Any
from typing import Dict
from typing import List
from typing import Optional
from typing import Tuple
from typing import Type
from typing import TypeVar

from typing_extensions import final

from apysc._type.revert_interface import RevertInterface

_SnapshotValue = TypeVar("_SnapshotValue")


class RevertMixIn(RevertInterface):
    _snapshot_exists_: Dict[str, bool]

    def _initialize_ss_exists_val_if_not_initialized(self) -> None:
        """
        Initialize _snapshot_exists_ value if this instance
        does not initialize it yet.
        """
        if hasattr(self, "_snapshot_exists_"):
            return
        self._snapshot_exists_ = {}

    @final
    def _run_all_make_snapshot_methods(self, *, snapshot_name: str) -> None:
        """
        Run all _make_snapshot methods. If an instance has
        multiple inheritances, and each class inherits
        RevertMixIn, each class calls the _make_snapshot
        methods.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self_type: Type = type(self)
        self._run_base_cls_make_snapshot_methods_recursively(
            class_=self_type, snapshot_name=snapshot_name
        )
        self._set_snapshot_exists_val(snapshot_name=snapshot_name)

    @final
    def _run_base_cls_make_snapshot_methods_recursively(
        self, *, class_: Type["RevertMixIn"], snapshot_name: str
    ) -> None:
        """
        Run base classes make_snapshot methods recursively.

        Parameters
        ----------
        class_ : type
            Target type.
        snapshot_name : str
            Target snapshot name.
        """
        base_classes: Tuple[Type, ...] = class_.__bases__
        for base_class in base_classes:
            if not issubclass(base_class, RevertMixIn):
                continue
            base_class._make_snapshot(self, snapshot_name=snapshot_name)
            self._run_base_cls_make_snapshot_methods_recursively(
                class_=base_class, snapshot_name=snapshot_name
            )
        class_._make_snapshot(self, snapshot_name=snapshot_name)

    @final
    def _run_all_revert_methods(self, *, snapshot_name: str) -> None:
        """
        Run all _revert methods. If an instance has multiple
        inheritances, and each class inherits RevertMixIn,
        each class calls the _revert methods.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self_type: Type = type(self)
        self._run_base_cls_revert_methods_recursively(
            class_=self_type, snapshot_name=snapshot_name
        )
        self._delete_snapshot_exists_val(snapshot_name=snapshot_name)

    @final
    def _run_base_cls_revert_methods_recursively(
        self, *, class_: Type["RevertMixIn"], snapshot_name: str
    ) -> None:
        """
        Run base classes revert methods recursively.

        Parameters
        ----------
        class_ : type
            Target type.
        snapshot_name : str
            Target snapshot name.
        """
        base_classes: Tuple[Type, ...] = class_.__bases__
        for base_class in base_classes:
            if not issubclass(base_class, RevertMixIn):
                continue
            base_class._revert(self, snapshot_name=snapshot_name)
            self._run_base_cls_revert_methods_recursively(
                class_=base_class, snapshot_name=snapshot_name
            )
        class_._revert(self, snapshot_name=snapshot_name)

    @final
    def _snapshot_exists(self, *, snapshot_name: str) -> bool:
        """
        Get a boolean value whether a snapshot value exists or not.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.

        Returns
        -------
        snapshot_exists : bool
            Boolean value, whether a snapshot value exists or not.
        """
        self._initialize_ss_exists_val_if_not_initialized()
        return snapshot_name in self._snapshot_exists_

    @final
    def _set_snapshot_exists_val(self, *, snapshot_name: str) -> None:
        """
        Set a boolean value whether a snapshot value exists or not.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_ss_exists_val_if_not_initialized()
        self._snapshot_exists_[snapshot_name] = True

    @final
    def _delete_snapshot_exists_val(self, *, snapshot_name: str) -> None:
        """
        Delete a boolean value whether a snapshot value exists or not.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_ss_exists_val_if_not_initialized()
        if snapshot_name in self._snapshot_exists_:
            del self._snapshot_exists_[snapshot_name]

    @final
    def _get_next_snapshot_name(self) -> str:
        """
        Get the next snapshot name.

        Returns
        -------
        snapshot_name : str
            Next snapshot name.
        """
        from apysc._expression import expression_variables_util
        from apysc._expression.var_names import SNAPSHOT

        snapshot_name: str = expression_variables_util.get_next_variable_name(
            type_name=SNAPSHOT
        )
        return snapshot_name

    @final
    def _set_single_snapshot_val_to_dict(
        self, *, dict_name: str, value: Any, snapshot_name: str
    ) -> None:
        """
        Set a single snapshot value to the specified
        name dictionary.

        Notes
        -----
        If a snapshot value of the same name already exists,
        this interface stops the method execution.

        Parameters
        ----------
        dict_name : str
            Target dictionary name.
        value : Any
            Target value.
        snapshot_name : str
            Target snapshot name.
        """
        if not hasattr(self, dict_name) or getattr(self, dict_name) is None:
            setattr(self, dict_name, {})
        if self._snapshot_exists(snapshot_name=snapshot_name):
            return
        target_dict: Optional[Dict[str, Any]] = getattr(self, dict_name)
        if target_dict is None:
            target_dict = {}
        target_dict[snapshot_name] = value

    @final
    def _get_snapshot_val_if_exists(
        self,
        *,
        current_value: _SnapshotValue,
        snapshot_dict: Optional[Dict[str, _SnapshotValue]],
        snapshot_name: str,
    ) -> _SnapshotValue:
        """
        Get a snapshot value from a specified dictionary if a snapshot exists.

        Parameters
        ----------
        current_value : _SnapshotValue
            A current value.
        snapshot_dict : Optional[Dict[str, _SnapshotValue]]
            A dictionary that contains a snapshot value.
        snapshot_name : str
            A target snapshot name.

        Returns
        -------
        snapshot_value : _SnapshotValue
            A snapshot value. If a snapshot value does not exist,
            this interface returns a current value.
        """
        if snapshot_dict is None:
            snapshot_dict = {}
        if not self._snapshot_exists(snapshot_name=snapshot_name):
            return current_value
        snapshot_value: _SnapshotValue = snapshot_dict[snapshot_name]
        return snapshot_value


def make_snapshots_of_each_scope_vars(
    *, locals_: Dict[str, Any], globals_: Dict[str, Any]
) -> str:
    """
    Make snapshots of each scope's variables.

    Parameters
    ----------
    locals_ : dict
        Local scope's variables.
    globals_ : dict
        Global scope's variables.

    Returns
    -------
    snapshot_name : str
        Snapshot name to be used.
    """
    variables: List[Any] = [*locals_.values(), *globals_.values()]
    snapshot_name: str = make_variables_snapshots(variables=variables)
    return snapshot_name


def make_variables_snapshots(*, variables: List[Any]) -> str:
    """
    Make snapshots of specified variables.

    Parameters
    ----------
    variables : list
        Variables to make snapshots.

    Returns
    -------
    snapshot_name : str
        Snapshot name to be used.
    """
    ended: Dict[int, bool] = {}
    snapshot_name: str = ""
    for variable in variables:
        if not isinstance(variable, RevertMixIn):
            continue
        var_id: int = id(variable)
        if var_id in ended:
            continue
        if snapshot_name == "":
            snapshot_name = variable._get_next_snapshot_name()
        variable._run_all_make_snapshot_methods(snapshot_name=snapshot_name)
        ended[var_id] = True
    return snapshot_name


def revert_each_scope_vars(
    *, snapshot_name: str, locals_: Dict[str, Any], globals_: Dict[str, Any]
) -> None:
    """
    Revert each scope's variables.

    Parameters
    ----------
    snapshot_name : str
        Snapshot name to use.
    locals_ : dict
        Local scope's variables.
    globals_ : dict
        Global scope's variables.
    """
    variables: List[Any] = [*locals_.values(), *globals_.values()]
    revert_variables(snapshot_name=snapshot_name, variables=variables)


def revert_variables(*, snapshot_name: str, variables: List[Any]) -> None:
    """
    Revert each variable.

    Parameters
    ----------
    snapshot_name : str
        Snapshot name to use.
    variables : list
        Each variable to revert.
    """
    ended: Dict[int, bool] = {}
    for variable in variables:
        if not isinstance(variable, RevertMixIn):
            continue
        var_id: int = id(variable)
        if var_id in ended:
            continue
        variable._run_all_revert_methods(snapshot_name=snapshot_name)
        ended[var_id] = True



"""
This module is for the common utilities and definitions
for the documents.
"""

import os
from typing import List

from apysc._lint_and_doc.docs_lang import Lang


def _get_src_dir_path() -> str:
    """
    Get a source directory path.

    Returns
    -------
    src_dir_path : str
        A source directory path.
    """
    src_dir_path: str = "/mnt/apysc/docs_src/source/"
    if os.path.isdir("./docs_src/"):
        src_dir_path = "./docs_src/source/"
    return src_dir_path


def get_docs_md_file_paths() -> List[str]:
    """
    Get documents' markdown file paths.

    Returns
    -------
    file_paths : List[str]
        Documenet's markdown file paths.
    """
    src_dir_path: str = _get_src_dir_path()
    file_names: List[str] = os.listdir(src_dir_path)
    file_paths: List[str] = []
    for file_name in file_names:
        file_path: str = os.path.join(src_dir_path, file_name)
        if not file_path.endswith(".md"):
            continue
        file_paths.append(file_path)
    return file_paths


def get_exclude_patterns(*, lang: Lang) -> List[str]:
    """
    Get excluding patterns' list.

    Parameters
    ----------
    lang : Lang
        A target language of documents.

    Returns
    -------
    patterns : List[str]
        Excluding patterns' list.
    """
    patterns: List[str] = []
    if lang == Lang.EN:
        for lang in Lang:
            patterns.append(f"{lang.value}_*.md")
        return patterns

    docs_file_paths: List[str] = get_docs_md_file_paths()
    for doc_file_path in docs_file_paths:
        basename: str = os.path.basename(doc_file_path)
        if not basename.startswith(f"{lang.value}_"):
            patterns.append(f"{basename}")
    return patterns



"""The mix-in class implementation for the `SvgText`'s
`_skip_fill_alpha_expression_appending` attribute.
"""

from typing import Optional
from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.number import Number


class SvgTextSkipFillAlphaExpAppendingMixIn:
    _skip_fill_alpha_expression_appending: bool = False

    @final
    @add_debug_info_setting(module_name=__name__)
    def _set_fill_alpha_expression_skipping_attr(
        self,
        *,
        fill_alpha: Optional[Union[float, Number]],
    ) -> None:
        """
        Set a boolean attribute, whether to skip a fill-alpha expression's appending
        or not.

        Parameters
        ----------
        fill_alpha : Optional[Union[float, Number]]
            A fill-alpha setting.
        """
        if fill_alpha is None:
            self._skip_fill_alpha_expression_appending = True
            return
        self._skip_fill_alpha_expression_appending = False



"""The center y geometry mix-in implementation for the `RectangleGeom`.
"""

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.number import Number


class RectangleGeomCenterYMixIn:
    _center_y: Number

    @property
    @add_debug_info_setting(module_name=__name__)
    def center_y(self) -> Number:
        """
        Get the rectangle center y coordinate.

        Returns
        -------
        center_y : Number
            The rectangle center y coordinate.
        """
        return self._center_y



"""Class implementation for the `visible` mix-in.
"""

from typing import Dict
from typing import Optional

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.attr_linking_mixin import AttrLinkingMixIn
from apysc._type.boolean import Boolean
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.variable_name_mixin import VariableNameMixIn
from apysc._type.variable_name_suffix_attr_or_var_mixin import (
    VariableNameSuffixAttrOrVarMixIn,
)
from apysc._validation import arg_validation_decos


class VisibleMixIn(
    VariableNameSuffixAttrOrVarMixIn,
    VariableNameMixIn,
    RevertMixIn,
    AttrLinkingMixIn,
):
    _visible: Boolean

    @final
    def _initialize_visible_if_not_initialized(self) -> None:
        """
        Initialize _visible attribute if this instance does not
        initialize it yet.
        """
        if hasattr(self, "_visible"):
            return
        suffix: str = self._get_attr_or_variable_name_suffix(value_identifier="visible")
        self._visible = Boolean(
            True,
            variable_name_suffix=suffix,
            skip_init_substitution_expression_appending=True,
        )

        self._append_visible_attr_linking_setting()

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_visible_attr_linking_setting(self) -> None:
        """
        Append a visible attribute linking settings.
        """
        self._append_applying_new_attr_val_exp(
            new_attr=self._visible, attr_name="visible"
        )
        self._append_attr_to_linking_stack(attr=self._visible, attr_name="visible")

    @property
    @add_debug_info_setting(module_name=__name__)
    def visible(self) -> Boolean:
        """
        Get a visibility value of this instance.

        Returns
        -------
        result : Boolean
            If this instance is visible, this interface returns True.

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color=ap.Color("#0af"))
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50
        ... )
        >>> rectangle.visible = ap.Boolean(False)
        >>> rectangle.visible
        Boolean(False)
        """
        from apysc._type import value_util

        self._initialize_visible_if_not_initialized()
        return value_util.get_copy(value=self._visible)

    @visible.setter
    @arg_validation_decos.is_apysc_boolean(arg_position_index=1)
    @add_debug_info_setting(module_name=__name__)
    def visible(self, value: Boolean) -> None:
        """
        Update a visibility value of this instance.

        Parameters
        ----------
        value : Boolean
            Boolean value to set.
        """
        self._visible = value
        self._append_visible_update_expression()

        self._append_visible_attr_linking_setting()

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_visible_update_expression(self) -> None:
        """
        Append visible property updating expression.
        """
        from apysc._expression import expression_data_util

        expression: str = (
            f"if ({self._visible.variable_name}) {{"
            f"\n  {self.variable_name}.show();"
            "\n}else {"
            f"\n  {self.variable_name}.hide();"
            "\n}"
        )
        expression_data_util.append_js_expression(expression=expression)

    _visible_snapshots: Optional[Dict[str, bool]] = None

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make value's snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_visible_if_not_initialized()
        self._set_single_snapshot_val_to_dict(
            dict_name="_visible_snapshots",
            value=self._visible._value,
            snapshot_name=snapshot_name,
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert value is snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_visible_if_not_initialized()
        self._visible._value = self._get_snapshot_val_if_exists(
            current_value=self._visible._value,
            snapshot_dict=self._visible_snapshots,
            snapshot_name=snapshot_name,
        )



"""The class implementation for the outlined `subtitles_off` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialSubtitlesOffOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `subtitles_off` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><g><path d="M20,4H6.83l2,2H20v11.17l1.76,1.76C21.91,18.65,22,18.34,22,18V6C22,4.9,21.1,4,20,4z"/><polygon points="18,10 12.83,10 14.83,12 18,12"/><path d="M1.04,3.87l1.2,1.2C2.09,5.35,2,5.66,2,6v12c0,1.1,0.9,2,2,2h13.17l2.96,2.96l1.41-1.41L2.45,2.45L1.04,3.87z M4,6.83 L7.17,10H6v2h2v-1.17L11.17,14H6v2h7.17l2,2H4V6.83z"/></g></g></svg>'  # noqa



"""Class implementation for the SVG text's font-family mix-in.
"""

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.array import Array
from apysc._type.string import String
from apysc._type.variable_name_mixin import VariableNameMixIn
from apysc._validation import arg_validation_decos


class SvgTextFontFamilyMixIn(
    VariableNameMixIn,
):
    @property
    @add_debug_info_setting(module_name=__name__)
    def font_family(self) -> Array[String]:
        """
        Get a current font-family settings.

        Returns
        -------
        font_family : Array[String]
            A current font-family settings.
            Each string in an array contains a font name (e.g., `Times New Roman`).
        """
        from apysc._type.variable_name_suffix_utils import (
            get_attr_or_variable_name_suffix,
        )

        suffix: str = get_attr_or_variable_name_suffix(
            instance=self,
            value_identifier="font_family",
        )
        font_family_string: String = String("", variable_name_suffix=suffix)
        self._append_font_family_string_getter_expression(
            font_family_string=font_family_string
        )

        font_family: Array[String] = font_family_string.split(sep=String(","))
        return font_family

    @font_family.setter
    @add_debug_info_setting(module_name=__name__)
    @arg_validation_decos.is_apysc_string_array(arg_position_index=1, optional=False)
    def font_family(self, value: Array[String]) -> None:
        """
        Set a font-family settings.

        Parameters
        ----------
        value : Array[String]
            A font-family setting.
            Each string in an array needs to be a font name (e.g., `Times New Roman`).
        """
        from apysc._expression import expression_data_util

        font_family_str: String = value.join(sep=String(","))
        expression: str = (
            f'{self.variable_name}.font({{"family": {font_family_str.variable_name}}});'
        )
        expression_data_util.append_js_expression(expression=expression)

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_font_family_string_getter_expression(
        self,
        *,
        font_family_string: String,
    ) -> None:
        """
        Append a font-family string's getter expression string.

        Parameters
        ----------
        font_family_string : String
            _description_
        """
        from apysc._expression import expression_data_util

        expression: str = (
            f"{font_family_string.variable_name} = "
            f'{self.variable_name}.font("family");'
        )
        expression_data_util.append_js_expression(expression=expression)



"""The class implementation for the outlined `closed_caption_disabled` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialClosedCaptionDisabledOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `closed_caption_disabled` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><rect fill="none" height="24" width="24"/><path d="M13,10c0-0.55,0.45-1,1-1h3c0.55,0,1,0.45,1,1v1h-1.5v-0.5h-2v1L13,10z M16.5,13.5l1.21,1.21C17.89,14.52,18,14.27,18,14v-1 h-1.5V13.5z M8.83,6H19v10.17l1.98,1.98c0-0.05,0.02-0.1,0.02-0.16V6c0-1.1-0.9-2-2-2H6.83L8.83,6z M19.78,22.61L17.17,20H5 c-1.11,0-2-0.9-2-2V6c0-0.05,0.02-0.1,0.02-0.15L1.39,4.22l1.41-1.41l18.38,18.38L19.78,22.61z M7.5,13.5h2V13h0.67l-2.5-2.5H7.5 V13.5z M15.17,18L11,13.83V14c0,0.55-0.45,1-1,1H7c-0.55,0-1-0.45-1-1v-4c0-0.32,0.16-0.59,0.4-0.78L5,7.83V18H15.17z"/></svg>'  # noqa



"""Mix-in class implementation for the `_set_initial_background_fill_color`
method.
"""

from typing_extensions import final

from apysc._color.color import Color
from apysc._html.debug_mode import add_debug_info_setting
from apysc._validation import arg_validation_decos


class SetInitialBackgroundFillColorMixIn:
    _background_fill_color: Color

    @final
    @arg_validation_decos.is_color(arg_position_index=1, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def _set_initial_background_fill_color(
        self,
        *,
        background_fill_color: Color,
    ) -> None:
        """
        Set a chart's initial background fill-color.

        Parameters
        ----------
        background_fill_color : Color
            A chart's initial background fill-color.
        """
        self._background_fill_color = background_fill_color



"""Class implementation for the min-related mix-in.
"""

from typing import Any
from typing import List
from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.array import Array
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._validation import arg_validation_decos


class MinMixIn:
    @classmethod
    @final
    @add_debug_info_setting(module_name=__name__)
    @arg_validation_decos.is_nums_array(arg_position_index=1)
    def min(
        cls,
        values: Array,
    ) -> Number:
        """
        Get a minimum number from a specified array's values.

        Parameters
        ----------
        values : Array[Union[Int, Number, int, float]]
            An array of numbers.

        Returns
        -------
        min_value : Number
            Minimum number in an array.

        References
        ----------
        - Math min interface
            - https://simon-ritchie.github.io/apysc/en/math_min.html

        Examples
        --------
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> arr: ap.Array = ap.Array([10, 9.5, ap.Int(8), ap.Number(8.5)])
        >>> min_value: ap.Number = ap.Math.min(values=arr)
        >>> min_value
        Number(8.0)
        """
        from apysc._expression import expression_data_util

        min_value_variable_name_suffix: str = (
            _get_min_value_variable_name_suffix_from_arr(arr=values)
        )
        min_value: Number = Number(
            0,
            variable_name_suffix=min_value_variable_name_suffix,
        )
        min_float_value: float = _get_min_float_value(values=values)
        min_value._value = min_float_value
        expression: str = (
            f"{min_value.variable_name} = {values.variable_name}.reduce("
            "function (a, b) {return Math.min(a, b)});"
        )
        expression_data_util.append_js_expression(expression=expression)
        return min_value


def _get_min_value_variable_name_suffix_from_arr(*, arr: Array) -> str:
    """
    Get a minimum value's variable name suffix from a specified array.

    Parameters
    ----------
    arr : Array
        An array of numbers.

    Returns
    -------
    suffix : str
        An extracted variable name suffix.
    """
    from apysc._type.variable_name_suffix_mixin import VariableNameSuffixMixIn

    values: List[float] = [float(value) for value in arr._value]
    min_value: float = min(values)
    min_value_index: int = values.index(min_value)
    min_value_: Any = arr._value[min_value_index]
    if isinstance(min_value_, VariableNameSuffixMixIn):
        return min_value_._variable_name_suffix
    return ""


def _get_min_float_value(*, values: Array[Union[Int, Number, int, float]]) -> float:
    """
    Get a minimum float value from a specified array.

    Parameters
    ----------
    values : Array[Union[Int, Number, int, float]]
        An array of numbers.

    Returns
    -------
    min_value : float
        A minimum float value.
    """
    values_: List[float] = []
    for value in values._value:
        if isinstance(value, (Int, Number)):
            values_.append(float(value._value))
            continue
        values_.append(float(value))
    min_value: float = min(values_)
    return min_value



"""This module is for the translation mapping data of the
following document:

Document file: colorless.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# COLORLESS constant": "# COLORLESS 定数",
    ##################################################
    "This page explains the `COLORLESS` constant.": "このページでは `COLORLESS` 定数について説明します。",
    ##################################################
    "## What constant is this?": "## 定数の概要",
    ##################################################
    "The `COLORLESS` constant is the no-color setting.": "`COLORLESS`定数は無色の指定用の設定です。",
    ##################################################
    "If you set this constant to each color-related argument or property, the `apysc` clears its color.": "もし各色関係の引数や属性にこの定数を設定した場合、`apysc`ライブラリではその色の指定箇所で色を削除します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `COLORLESS` constant is a subclass of the `Color` class.": "`COLORLESS`定数は`Color`クラスのサブクラスです。",  # noqa
    ##################################################
    "So, you can specify its constant to each color-related argument or property.": "そのため、色関係の引数や属性にこの定数を指定することができます。",  # noqa
    ##################################################
    "The `apysc` displays no color since the `fill_color` and `line_color` argument's values are the `COLORLESS` constant in the following example:": "以下の例では`fill_color`と`line_color`引数の各値に`COLORLESS`定数が指定されているため`apysc`ライブラリでは色を表示しません:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n_ = ap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n_ = ap.Rectangle(\n    x=50,\n    y=50,\n    width=50,\n    height=50,\n    fill_color=ap.COLORLESS,\n    line_color=ap.COLORLESS,\n    line_thickness=3,\n)\n\nap.save_overall_html(dest_dir_path="./colorless_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\n_ = ap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n_ = ap.Rectangle(\n    x=50,\n    y=50,\n    width=50,\n    height=50,\n    fill_color=ap.COLORLESS,\n    line_color=ap.COLORLESS,\n    line_thickness=3,\n)\n\nap.save_overall_html(dest_dir_path="./colorless_basic_usage/")\n```',  # noqa
}



"""The class implementation for the outlined `view_sidebar` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialViewSidebarOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `view_sidebar` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/><g><path d="M2,4v16h20V4H2z M20,8.67h-2.5V6H20V8.67z M17.5,10.67H20v2.67h-2.5V10.67z M4,6h11.5v12H4V6z M17.5,18v-2.67H20V18H17.5z"/></g></g></svg>'  # noqa



"""The class implementation for the outlined `stop_screen_share` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialStopScreenShareOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `stop_screen_share` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M21.79 18l2 2H24v-2h-2.21zM1.11 2.98l1.55 1.56c-.41.37-.66.89-.66 1.48V16c0 1.1.9 2 2.01 2H0v2h18.13l2.71 2.71 1.41-1.41L2.52 1.57 1.11 2.98zM4 6.02h.13l4.95 4.93C7.94 12.07 7.31 13.52 7 15c.96-1.29 2.13-2.08 3.67-2.46l3.46 3.48H4v-10zm16 0v10.19l1.3 1.3c.42-.37.7-.89.7-1.49v-10c0-1.11-.9-2-2-2H7.8l2 2H20zm-7.07 3.13l2.79 2.78 1.28-1.2L13 7v2.13l-.07.02z"/></svg>'  # noqa



"""The class implementation for the outlined `games` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialGamesOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `games` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M13 4v2.67l-1 1-1-1V4h2m7 7v2h-2.67l-1-1 1-1H20M6.67 11l1 1-1 1H4v-2h2.67M12 16.33l1 1V20h-2v-2.67l1-1M15 2H9v5.5l3 3 3-3V2zm7 7h-5.5l-3 3 3 3H22V9zM7.5 9H2v6h5.5l3-3-3-3zm4.5 4.5l-3 3V22h6v-5.5l-3-3z"/></svg>'  # noqa



"""The class implementation for the `gif` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialGifIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `gif` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24" x="0"/></g><g><g><rect height="6" width="1.5" x="11.5" y="9"/><path d="M9,9H6c-0.6,0-1,0.5-1,1v4c0,0.5,0.4,1,1,1h3c0.6,0,1-0.5,1-1v-2H8.5v1.5h-2v-3H10V10C10,9.5,9.6,9,9,9z"/><polygon points="19,10.5 19,9 14.5,9 14.5,15 16,15 16,13 18,13 18,11.5 16,11.5 16,10.5"/></g></g></svg>'  # noqa



"""The class implementation for the outlined `extension` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialExtensionOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `extension` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M10.5 4.5c.28 0 .5.22.5.5v2h6v6h2c.28 0 .5.22.5.5s-.22.5-.5.5h-2v6h-2.12c-.68-1.75-2.39-3-4.38-3s-3.7 1.25-4.38 3H4v-2.12c1.75-.68 3-2.39 3-4.38 0-1.99-1.24-3.7-2.99-4.38L4 7h6V5c0-.28.22-.5.5-.5m0-2C9.12 2.5 8 3.62 8 5H4c-1.1 0-1.99.9-1.99 2v3.8h.29c1.49 0 2.7 1.21 2.7 2.7s-1.21 2.7-2.7 2.7H2V20c0 1.1.9 2 2 2h3.8v-.3c0-1.49 1.21-2.7 2.7-2.7s2.7 1.21 2.7 2.7v.3H17c1.1 0 2-.9 2-2v-4c1.38 0 2.5-1.12 2.5-2.5S20.38 11 19 11V7c0-1.1-.9-2-2-2h-4c0-1.38-1.12-2.5-2.5-2.5z"/></svg>'  # noqa



"""Class implementation for the `String` class's `split` mix-in.
"""

from typing import TYPE_CHECKING

from apysc._type.variable_name_mixin import VariableNameMixIn

if TYPE_CHECKING:
    from apysc._type.array import Array
    from apysc._type.string import String


class StringSplitMixIn(VariableNameMixIn):
    def split(self, *, sep: "String") -> "Array[String]":
        """
        Split a current string with a separator string.

        Parameters
        ----------
        sep : String
            A separator string.

        Returns
        -------
        splitted_strs : Array[String]
            A splitted strings' array.

        References
        ----------
        - String class split interface
            - https://simon-ritchie.github.io/apysc/en/string_split.html

        Examples
        --------
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> str_value: ap.String = ap.String("Lorem ipsum dolor sit")
        >>> splitted_strs: ap.Array[ap.String] = str_value.split(sep=ap.String(" "))
        >>> ap.assert_arrays_equal(splitted_strs, ["Lorem", "ipsum", "dolor", "sit"])
        """
        from apysc._type.array import Array
        from apysc._type.variable_name_suffix_utils import (
            get_attr_or_variable_name_suffix,
        )

        suffix: str = get_attr_or_variable_name_suffix(
            instance=self,
            value_identifier="splitted_strs",
        )
        splitted_strs: Array[String] = Array([], variable_name_suffix=suffix)
        self._append_split_expression(splitted_strs=splitted_strs, sep=sep)
        return splitted_strs

    def _append_split_expression(
        self,
        *,
        splitted_strs: "Array[String]",
        sep: "String",
    ) -> None:
        """
        Append a `split` method's expression string.

        Parameters
        ----------
        splitted_strs : Array[String]
            A splitted strings' array.
        sep : String
            A separator string.
        """
        from apysc._expression import expression_data_util

        expression: str = (
            f"{splitted_strs.variable_name} = "
            f"{self.variable_name}.split({sep.variable_name});"
        )
        expression_data_util.append_js_expression(expression=expression)



"""The class implementation for the `admin_panel_settings` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialAdminPanelSettingsIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `admin_panel_settings` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><g><path d="M17,11c0.34,0,0.67,0.04,1,0.09V6.27L10.5,3L3,6.27v4.91c0,4.54,3.2,8.79,7.5,9.82c0.55-0.13,1.08-0.32,1.6-0.55 C11.41,19.47,11,18.28,11,17C11,13.69,13.69,11,17,11z"/><path d="M17,13c-2.21,0-4,1.79-4,4c0,2.21,1.79,4,4,4s4-1.79,4-4C21,14.79,19.21,13,17,13z M17,14.38c0.62,0,1.12,0.51,1.12,1.12 s-0.51,1.12-1.12,1.12s-1.12-0.51-1.12-1.12S16.38,14.38,17,14.38z M17,19.75c-0.93,0-1.74-0.46-2.24-1.17 c0.05-0.72,1.51-1.08,2.24-1.08s2.19,0.36,2.24,1.08C18.74,19.29,17.93,19.75,17,19.75z"/></g></g></svg>'  # noqa



"""The mix-in class implementation for the `x_axis_column_name` value.
"""

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._validation import arg_validation_decos


class XAxisColumnNameMixIn:
    _x_axis_column_name: str

    @final
    @arg_validation_decos.is_builtin_string(arg_position_index=1, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def _set_initial_x_axis_column_name(
        self,
        *,
        x_axis_column_name: str,
    ) -> None:
        """
        Set an initial x-axis column name.

        Parameters
        ----------
        x_axis_column_name : str
            X-axis column name.
        """
        self._x_axis_column_name = x_axis_column_name



"""The class implementation for the `push_pin` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialPushPinIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `push_pin` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><path d="M16,9V4l1,0c0.55,0,1-0.45,1-1v0c0-0.55-0.45-1-1-1H7C6.45,2,6,2.45,6,3v0 c0,0.55,0.45,1,1,1l1,0v5c0,1.66-1.34,3-3,3h0v2h5.97v7l1,1l1-1v-7H19v-2h0C17.34,12,16,10.66,16,9z" fill-rule="evenodd"/></g></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: graphics_base_line_alpha.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# GraphicsBase line_alpha interface": "# GraphicsBase クラスの line_alpha インターフェイス",
    ##################################################
    "This page explains the `GraphicsBase` class `line_alpha` property interface.": "このページでは`GraphicsBase`クラスの`line_alpha`属性のインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `line_alpha` property interface updates or gets the instance's line alpha (opacity).": "`line_alpha`属性のインターフェイスではインスタンスの線の透明度の更新や取得を行うことができます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The getter or setter interface value becomes (or requires) the `Number` value (0.0 to 1.0).": "getterとsetterの両方のインターフェイスの値は`Number`型の0.0～1.0の範囲の値となります。",  # noqa
    ##################################################
    "The following example sets the 0.5 line alpha to the second rectangle and 0.25 to the third rectangle:": "以下のコード例では0.5の線の透明度を2番目の四角に設定し、0.25の線の透明度を3番目の四角に設定しています:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=350,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.line_style(color=ap.Color("#0af"), thickness=5)\n\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(x=150, y=50, width=50, height=50)\nrectangle_2.line_alpha = ap.Number(0.5)\n\nrectangle_3: ap.Rectangle = sprite.graphics.draw_rect(x=250, y=50, width=50, height=50)\nrectangle_3.line_alpha = ap.Number(0.25)\n\nap.save_overall_html(dest_dir_path="./graphics_base_line_alpha_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=350,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.line_style(color=ap.Color("#0af"), thickness=5)\n\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(x=150, y=50, width=50, height=50)\nrectangle_2.line_alpha = ap.Number(0.5)\n\nrectangle_3: ap.Rectangle = sprite.graphics.draw_rect(x=250, y=50, width=50, height=50)\nrectangle_3.line_alpha = ap.Number(0.25)\n\nap.save_overall_html(dest_dir_path="./graphics_base_line_alpha_basic_usage/")\n```',  # noqa
    ##################################################
    "## line_alpha property API": "## line_alpha 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get this instance's line alpha (opacity).<hr>": "インスタンスの線の透明度を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `line_alpha`: Number": "- `line_alpha`: Number",
    ##################################################
    "  - Current line alpha (opacity. 0.0 to 1.0).": "  - 現在の線の透明度（0.0～1.0）。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=5, alpha=1.0)\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> rectangle.line_alpha = ap.Number(0.5)\n>>> rectangle.line_alpha\nNumber(0.5)\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=5, alpha=1.0)\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> rectangle.line_alpha = ap.Number(0.5)\n>>> rectangle.line_alpha\nNumber(0.5)\n```',  # noqa
}



"""This module is for the `delete` function's expression
implementation.
"""

from apysc._type.variable_name_mixin import VariableNameMixIn
from apysc._validation import arg_validation_decos


@arg_validation_decos.is_variable_name_interface_type(arg_position_index=0)
def delete(value: VariableNameMixIn) -> None:
    """
    Delete a specified value and make it undefined.

    Parameters
    ----------
    value : VariableNameMixIn
        A target value to delete.
    """
    _remove_from_parent(value=value)
    value._is_deleted_object = True
    _append_delete_expression(value=value)


def _remove_from_parent(*, value: VariableNameMixIn) -> None:
    """
    Remove a specified value from a parent if it is the `ParentMixIn`
    instance.

    Parameters
    ----------
    value : VariableNameMixIn
        _description_
    """
    from apysc._display.parent_mixin import ParentMixIn

    if not isinstance(value, ParentMixIn):
        return
    value.remove_from_parent()


def _append_delete_expression(*, value: VariableNameMixIn) -> None:
    """
    Append a delete expression.

    Parameters
    ----------
    value : VariableNameMixIn
        A target value to delete.
    """
    from apysc._expression import expression_data_util

    expression: str = (
        f"delete {value.variable_name};" f"\n{value.variable_name} = undefined;"
    )
    expression_data_util.append_js_expression(expression=expression)



"""The class implementation for the outlined `explore_off` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialExploreOffOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `explore_off` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M12 4c4.41 0 8 3.59 8 8 0 1.48-.41 2.86-1.12 4.06l1.46 1.46C21.39 15.93 22 14.04 22 12c0-5.52-4.48-10-10-10-2.04 0-3.93.61-5.51 1.66l1.46 1.46C9.14 4.41 10.52 4 12 4zm2.91 8.08L17.5 6.5l-5.58 2.59 2.99 2.99zM2.1 4.93l1.56 1.56C2.61 8.07 2 9.96 2 12c0 5.52 4.48 10 10 10 2.04 0 3.93-.61 5.51-1.66l1.56 1.56 1.41-1.41L3.51 3.51 2.1 4.93zm3.02 3.01l3.98 3.98-2.6 5.58 5.58-2.59 3.98 3.98c-1.2.7-2.58 1.11-4.06 1.11-4.41 0-8-3.59-8-8 0-1.48.41-2.86 1.12-4.06z"/></svg>'  # noqa



"""The mix-in class implementation for the `to_string` method.
"""

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.string import String
from apysc._type.variable_name_mixin import VariableNameMixIn


class ToStringMixIn(VariableNameMixIn):
    @final
    @add_debug_info_setting(module_name=__name__)
    def to_string(self) -> String:
        """
        Convert this instance to a string.

        Returns
        -------
        string : String
            A converted string.

        References
        ----------
        - to_string interface
            - https://simon-ritchie.github.io/apysc/en/to_string.html

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage(
        ...     background_color=ap.Color("#333"), stage_width=200, stage_height=200
        ... )
        >>> int_value: ap.Int = ap.Int(value=100)
        >>> string: ap.String = int_value.to_string()
        >>> ap.assert_equal(string, "100")
        """
        from apysc._expression import expression_data_util
        from apysc._type.variable_name_suffix_utils import (
            get_attr_or_variable_name_suffix,
        )

        suffix: str = get_attr_or_variable_name_suffix(
            instance=self, value_identifier="string"
        )
        string: String = String("", variable_name_suffix=suffix)
        expression: str = f"{string.variable_name} = String({self.variable_name});"
        expression_data_util.append_js_expression(expression=expression)
        return string



"""The class implementation for the outlined `domain_verification` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialDomainVerificationOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `domain_verification` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><g><polygon points="16.6,10.88 15.18,9.46 10.94,13.71 8.82,11.58 7.4,13 10.94,16.54"/><path d="M19,4H5C3.89,4,3,4.9,3,6v12c0,1.1,0.89,2,2,2h14c1.1,0,2-0.9,2-2V6C21,4.9,20.11,4,19,4z M19,18H5V8h14V18z"/></g></g></svg>'  # noqa



"""The class implementation for the `call_end` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialCallEndIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `call_end` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M12 9c-1.6 0-3.15.25-4.6.72v3.1c0 .39-.23.74-.56.9-.98.49-1.87 1.12-2.66 1.85-.18.18-.43.28-.7.28-.28 0-.53-.11-.71-.29L.29 13.08c-.18-.17-.29-.42-.29-.7 0-.28.11-.53.29-.71C3.34 8.78 7.46 7 12 7s8.66 1.78 11.71 4.67c.18.18.29.43.29.71 0 .28-.11.53-.29.71l-2.48 2.48c-.18.18-.43.29-.71.29-.27 0-.52-.11-.7-.28-.79-.74-1.69-1.36-2.67-1.85-.33-.16-.56-.5-.56-.9v-3.1C15.15 9.25 13.6 9 12 9z"/></svg>'  # noqa
